function [hme] = hmeCreateMixture(children,d,param)
% [hme] = hmeCreateMixture(children,b,d,param)
%
% Create HME model consisting of a mixture of a set of HME models.
% The model parameters are initialized to zero if not provided.
%
% INPUTS
% 	children 	1xb cell array of HME models
%	d		dimensionality of data
%	[param]		dxb matrix of mixing model coefficients
%
% OUTPUTS
%	hme		HME model
%
% See also hmeCreateExpert, hmeInitRand, hmeInitLR. 
%
% David Martin <dmartin@eecs.berkeley.edu> 
% Charless Fowlkes <fowlkes@eecs.berkeley.edu>
% May 7, 2002

% Copyright (C) 2002 David R. Martin <dmartin@eecs.berkeley.edu>
% Copyright (C) 2002 Charless C. Fowlkes <fowlkes@eecs.berkeley.edu>
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License as
% published by the Free Software Foundation; either version 2 of the
% License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
% 02111-1307, USA, or see http://www.gnu.org/copyleft/gpl.html.

error(nargchk(2,3,nargin));

b = length(children);

hme.leaf = 0;
hme.param = zeros(d,b);
hme.children = children;

if nargin == 3,
  hme.param = param;
end

