%DEMGLM1 Demonstrate simple classification using a generalized linear model.
%
%	Description
%	 The problem consists of a two dimensional input matrix DATA and a
%	vector of classifications T.  The data is  generated from two
%	Gaussian clusters, and a generalized linear model with logistic
%	output is trained using iterative reweighted least squares. A plot of
%	the data together with the 0.1, 0.5 and 0.9 contour lines of the
%	conditional probability is generated.
%
%	See also
%	DEMGLM2, GLM, GLMTRAIN
%

%	Copyright (c) Ian T Nabney (1996-2001)

if 0
    clc; clear all; close all;
    % Generate data from two classes in 2d
    input_dim = 2;
    
    % Fix seeds for reproducible results
    randn('state', 42);
    rand('state', 42);
    
    close all;
    
    ndata = 100;
    % Generate mixture of two Gaussians in two dimensional space
    mix = gmm(2, 2, 'spherical');
    mix.priors = [0.4 0.6];              % Cluster priors
    mix.centres = [2.0, 2.0; 0.0, 0.0];  % Cluster centres
    mix.covars = [1.0, 1.0];
    
    [data, label] = gmmsamp(mix, ndata);
    targets = label - ones(ndata, 1);
    
    save temp;
    return;
end

clear all; clc; close all; load temp;

fh1 = figure;
plot(data(label==1,1), data(label==1,2), 'bo');
hold on
axis([-4 5 -4 5])
set(gca, 'box', 'on')
plot(data(label==2,1), data(label==2,2), 'rx')
title('Data')

issoftmax = 0;

if issoftmax
    targets = [targets targets];
    targets(:,2)=1-targets(:,2);
    % two class outputs
    net = glm(input_dim, 2, 'softmax');
else
    net = glm(input_dim, 1, 'logistic');
end

% do not randomize intial condition
net = glmunpak(net, [ 0 0 0] );

options = foptions;
options(1) = 1; % display
options(14) = 100;
% net = glmtrain(net, options, data, targets);
weight = ones(ndata,1) * 1;
net = glmtrain_weight(net, options, data, weight, targets);


z = glmfwd(net, data);
z(z>=0.5)=1;
z(z<0.5)=0;

error = sum(z~=targets)

x = -4.0:0.2:5.0;
y = -4.0:0.2:5.0;
[X, Y] = meshgrid(x,y);
X = X(:);
Y = Y(:);
grid = [X Y];
Z = glmfwd(net, grid);
if issoftmax
    tZ = reshape(Z(:,2), length(x), length(y));
else
    tZ = reshape(Z(:), length(x), length(y));
end
v = [0.1 0.5 0.9];
[c, h] = contour(x, y, tZ, v);
title('Generalized Linear Model')
set(h, 'linewidth', 3)
clabel(c, h);

tmp = [net.w1; net.b1]
net.w1 / sum(net.w1)

