function [LL, Err] = CA_cond_LL(CA_model, type, ...
    train_X, train_C, train_A, condtype)
% NEWONE

% minimum p(C|X)
min_pc_x = 0.001;
% minimum p(A|X)
min_pa_x = 0.001;

assert(strcmpi(CA_model.type,type));

num = size(train_X, 1);

nclass = CA_model.nclass;
nafford = CA_model.nafford;
objmix = CA_model.objmix;

probC = CA_model.probC;
probA_C = CA_model.probA_C; % CA_chain.probA_C(i,j) -> P(A_j=1 | C=i)

pX_C = zeros(num, nclass);

LL = zeros(num,1);



%%%%%% for both CA_chain or CA_full, first obtain p(C|X)%%%%%%%%%
for i = 1:nclass
    pX_C(:,i) = gmmprob(objmix{i}, train_X);
end
pCX = pX_C .* (ones(num,1)*probC);

for k = 1:num
    this_pC_X = pCX(k,:) / sum(pCX(k,:));
    this_pC_X = this_pC_X + min_pc_x;
    pC_X(k,:) = this_pC_X / sum(this_pC_X);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pA_X = zeros(num,1);

Err = -1;


switch type
    case 'CA_chain'
        switch condtype
            case 'pA_X_group'
                % we have this option because in optimization, there
                % is no need to calculate the error, to save computations
                for k = 1:num
                    A = train_A(k,:);
                    this_pAC_X = pC_X(k,:);
                    for j = 1:nafford
                        if A(1,j) == 1
                            this_pAC_X = this_pAC_X .* probA_C(:,j)';
                        elseif A(1,j) == 0
                            this_pAC_X = this_pAC_X .* (1-probA_C(:,j)');
                        end
                    end
                    pA_X(k) = sum(this_pAC_X);
                end

                % addjust to > min_pa_x
                pA_X = max(pA_X, min_pa_x);
                LL = log(pA_X);
                LL = sum(LL);
                
                Err = 0;
            case 'pA_X_group_err'
                %% assert( any(train_A==-1) == 0);
                % the joint LL handles missing value too
                % but just that the joint error is too bad, because
                %    prediction can't be -1, therefore not match
                
                
                % the JOINT ERROR from joint argmax
                % first we calculate group error
                naffcomb = 2^nafford;
                joint_probA_C = - ones(naffcomb, nclass);
                ind = 1;
                joint_index = zeros(naffcomb, nafford);
                for i=0:1
                    for j=0:1
                        for k=0:1
                            mask = [i,j,k];
                            for obj = 1:nclass
                                joint_probA_C(ind,obj) = prod( probA_C(obj,:).^mask ...
                                    .* (1-probA_C(obj,:)).^(1-mask) );
                            end
                            joint_index(ind,:) = mask;
                            ind = ind+1;
                        end
                    end
                end

                % calculate the error
                for k = 1:num
                    for j = 1:naffcomb
                        joint_pAC_X(j) = sum(pC_X(k,:) .* joint_probA_C(j,:));
                    end
                    [maxjoint, maxid]=max(joint_pAC_X);
                    maxA(k,:) = joint_index(maxid,:);
                end
                diffA = (maxA ~= train_A);
                error = any(diffA');
                assert(length(error) == num);
                Err = sum(error) / num;

                % the following calculates LL
                % REAL CLL func, not separating each A^k
                for k = 1:num
                    A = train_A(k,:);
                    this_pAC_X = pC_X(k,:);
                    for j = 1:nafford
                        if A(1,j) == 1
                            this_pAC_X = this_pAC_X .* probA_C(:,j)';
                        elseif A(1,j) == 0
                            this_pAC_X = this_pAC_X .* (1-probA_C(:,j)');
                        end
                    end
                    pA_X(k) = sum(this_pAC_X);
                end

                % addjust to > min_pa_x
                pA_X = max(pA_X, min_pa_x);
                LL = log(pA_X);
                LL = sum(LL);
                
            case 'pA_X_sep' % CLL when separting each A
                % this one does not care whether C is observed or not
                matLL=ones(num,nafford);
                
                for k = 1:num
                    A = train_A(k,:);
                    for j = 1:nafford
                        if A(1,j) == 1
                            this_pAC_X = pC_X(k,:) .* probA_C(:,j)';
                        elseif A(1,j) == 0
                            this_pAC_X = pC_X(k,:) .* (1-probA_C(:,j)');
                        else
                            this_pAC_X = 0; %no use, the mask matrix filter these
                        end
                        pa_x = sum(this_pAC_X);
                        pa_x = max(pa_x, min_pa_x);
                        matLL(k,j)=pa_x;
                    end
                end
                %%%% IMPORTANT %%%%%%
                matLL(train_A==-1) = 1;
                LL=sum(log(matLL(:)));

                % now I can find the error_separate:
                % since this is CLL p(A^k=a^k|x_n), p < 0.5 is error
                % for both A^k = 1 or 0
                error = (matLL<0.5);
                mask = (train_A > -1); % known A value
                error = error.*mask;
                % Err(1:nafford) is the error rate for each affordance
                Err = sum(error) ./ sum(mask);
                % Err(nafford+1) is the error rate in general
                Err(1,nafford+1) = sum(error(:)) / sum(mask(:));

                % FAKE one, only applied to limited training data
                Err(1,nafford+2) = sum(any(error')) / num;
            case 'pAC_X'
                error(['Not implemented joint type ', jointtype]);

            otherwise
                error(['Unknown CA_chain joint type ', jointtype]);
        end

    case 'CA_full'
        clear c i j k;
        %%%%%%%%%%%%%%% (1) same as CA_posterior %%%%%%%%%%%%%%
        probA_C = [];
        affglm = CA_model.affglm;

        % glm_pA_CX stores the matrix of p(A_k|C,X)
        % each element stores for fixed A_k and C
        glm_pA_CX = cell(nclass, nafford);
        glm_mask = zeros(nclass, nafford);

        for c = 1:nclass
            for j = 1:nafford
                glmtmp = affglm{c,j};
                if isstruct(glmtmp)
                    tmpfwd = glmfwd(glmtmp, train_X);
                    glm_pA_CX{c,j} = tmpfwd;
                    % mark it as special, where classifier is involved
                    glm_mask(c,j) = 1;
                elseif isscalar(glmtmp) && glmtmp >=0 && glmtmp<=1
                    glm_pA_CX{c,j} = glmtmp;
                    % glm_mask(c,j) = 0 by default
                else
                    error(['Unknown glm type for: c,j=' num2str([c,j])]);
                end
            end
        end
        %%%%%%%%%%%%%%%  (1) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        clear c i j k;
        switch condtype
            case 'pA_X_group'
                LL=zeros(num,1);
                for k=1:num
                    %%%%%%%%%%%%%% (2) same as CA_posterior %%%%%%%%%%%%%%
                    A = train_A(k,:);

                    % p(A |C,X)
                    pa_cx = zeros(nclass,nafford);
                    for c=1:nclass
                        for j=1:nafford
                            % special case, use classifier response
                            if glm_mask(c,j) == 1
                                pa_cx(c,j) = glm_pA_CX{c,j}(k);
                            else % mask(i,j) == 0
                                pa_cx(c,j) = glm_pA_CX{c,j};
                            end
                        end
                    end
                    %%%%%%%%%%%%%%%  (2) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    
                    %% this part mimic CA_chain
                    this_pAC_X = pC_X(k,:);
                    for j = 1:nafford
                        if A(1,j) == 1
                            this_pAC_X = this_pAC_X .* pa_cx(:,j)';
                        elseif A(1,j) == 0
                            this_pAC_X = this_pAC_X .* (1-pa_cx(:,j)');
                        end
                    end
                    pa_x = sum(this_pAC_X);
                    LL(k) = pa_x;
                end
                LL = max(LL, min_pa_x);
                LL = sum(log(LL));
                
                Err = -1;
                
            case 'pA_X_sep'
                matLL=ones(num,nafford);
                for k=1:num
                    %%%%%%%%%%%%%% (2) same as CA_posterior %%%%%%%%%%%%%%
                    A = train_A(k,:);

                    % p(A |C,X)
                    pa_cx = zeros(nclass,nafford);
                    for c=1:nclass
                        for j=1:nafford
                            % special case, use classifier response
                            if glm_mask(c,j) == 1
                                pa_cx(c,j) = glm_pA_CX{c,j}(k);
                            else % mask(i,j) == 0
                                pa_cx(c,j) = glm_pA_CX{c,j};
                            end
                        end
                    end
                    %%%%%%%%%%%%%%%  (2) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    
                    %% this part mimic CA_chain
                    for j = 1:nafford
                        if A(1,j) == 1
                            this_pAC_X = pC_X(k,:) .* pa_cx(:,j)';
                        elseif A(1,j) == 0
                            this_pAC_X = pC_X(k,:) .* (1-pa_cx(:,j)');
                        else
                            this_pAC_X = -1; %no use, the mask matrix filter these
                        end
                        pa_x = sum(this_pAC_X);
                        pa_x = max(pa_x, min_pa_x);
                        matLL(k,j)=pa_x;
                    end
                end
                %%%% IMPORTANT %%%%%%
                matLL(train_A==-1) = 1;                
                LL=sum(log(matLL(:)));
                
                error = (matLL<0.5);
                mask = (train_A > -1); % known A value
                error = error.*mask;
                % Err(1:nafford) is the error rate for each affordance
                Err = sum(error) ./ sum(mask);
                % Err(nafford+1) is the error rate in general
                Err(1,nafford+1) = sum(error(:)) / sum(mask(:));

                % FAKE one, only applied to limited training data
                Err(1,nafford+2) = sum(any(error')) / num;

            otherwise
                error(['Unknown CA_full joint type ', jointtype]);
        end % end of CA_full
        
        
        
        
    otherwise %not CA_chain or CA_full
        error(['Unknown CA model type ', type]);
end

