function LL = CA_joint_LL(CA_model, type, train_X, train_C, train_A, jointtype)

assert(strcmpi(CA_model.type,type));

num = size(train_X, 1);

nclass = CA_model.nclass;
nafford = CA_model.nafford;
objmix = CA_model.objmix;

probC = CA_model.probC;
probA_C = CA_model.probA_C; % CA_chain.probA_C(i,j) -> P(A_j=1 | C=i)

pX_C = zeros(num, nclass);

LL = zeros(num,1);

for i = 1:nclass
    pX_C(:,i) = gmmprob(objmix{i}, train_X);
end
pCX = pX_C .* (ones(num,1)*probC);
% have pCX already

switch type
    case 'CA_chain'
        switch jointtype
            case 'pXCA'
                for k = 1:num
                    C = train_C(i);
                    A = train_A(i,:);
                    if C > 0 % C observed
                        ll = pCX(k,C);
                        
                        for j = 1:nafford
                            if A(1,j) == 1
                                ll = ll * probA_C(C,j);
                            elseif A(1,j) == 0
                                ll = ll * (1-probA_C(C,j));
                            end
                        end
                    else
                        ll = pCX(k,:);

                        for j = 1:nafford
                            if A(1,j) == 1
                                ll = ll .* probA_C(:,j)';
                            elseif A(1,j) == 0
                                ll = ll .* (1-probA_C(:,j)');
                            end
                        end
                        
                        % sum over C
                        ll = sum(ll);
                    end
                    LL(k) = ll;
                end
                
                % Note that we need to superimpose an eps
                LL = max(LL, 1e-30);
                LL = log(LL);
            case 'pXA' % ignore even if C is observed
                for k = 1:num
                    C = train_C(i);
                    A = train_A(i,:);
                    
                    ll = pCX(k,:);
                    
                    for j = 1:nafford
                        if A(1,j) == 1
                            ll = ll .* probA_C(:,j)';
                        elseif A(1,j) == 0
                            ll = ll .* (1-probA_C(:,j)');
                        end
                    end
                    
                    % sum over C
                    ll = sum(ll);
                    LL(k) = ll;
                end
                
                LL = max(LL, 1e-100);
                LL = log(LL);
                
            otherwise
                error(['Unknown joint type ', jointtype]);
        end
            
        
        
        
        
    case 'CA_full'
        probA_C = [];
        affglm = CA_model.affglm;

        % glm_pA_CX stores the matrix of p(A_k|C,X)
        % each element stores for fixed A_k and C
        glm_pA_CX = cell(nclass, nafford);
        glm_mask = zeros(nclass, nafford);

        for c = 1:nclass
            for j = 1:nafford
                glmtmp = affglm{c,j};
                if isstruct(glmtmp)
                    tmpfwd = glmfwd(glmtmp, train_X);
                    glm_pA_CX{c,j} = tmpfwd;
                    % mark it as special, where classifier is involved
                    glm_mask(c,j) = 1;
                elseif isscalar(glmtmp) && glmtmp >=0 && glmtmp<=1
                    glm_pA_CX{c,j} = glmtmp;
                    % glm_mask(c,j) = 0 by default
                else
                    error(['Unknown glm type for: c,j=' num2str([c,j])]);
                end
            end
        end
        
        switch jointtype
            case 'pXA'
                LL=ones(num,1);
                for k=1:num
                    %%%%%%%%%%%%%% (2) same as CA_posterior %%%%%%%%%%%%%%
                    A = train_A(k,:);

                    % p(A |C,X)
                    pa_cx = zeros(nclass,nafford);
                    for c=1:nclass
                        for j=1:nafford
                            % special case, use classifier response
                            if glm_mask(c,j) == 1
                                pa_cx(c,j) = glm_pA_CX{c,j}(k);
                            else % mask(i,j) == 0
                                pa_cx(c,j) = glm_pA_CX{c,j};
                            end
                        end
                    end
                    %%%%%%%%%%%%%%%  (2) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    
                    ll = pCX(k,:);
                    for j = 1:nafford
                        if A(1,j) == 1
                            ll = ll .* pa_cx(:,j)';
                        elseif A(1,j) == 0
                            ll = ll .* (1-pa_cx(:,j)');
                        end
                    end
                    ll = sum(ll);
                    LL(k) = ll;
                end
                LL = max(LL, 1e-100);
                LL = log(LL);
                
    
            otherwise
                error(['CA_full not implemented yet']);
        end
        
    otherwise
        error(['Unknown CA model type ', type]);
end









