function [post_pC, post_pA, pC_X]=CA_posterior(CA_model, type, ...
    train_X, train_C, train_A, min_pc_x)
% NEWONE
% CA: calculate the posterior probability of C and A from the CA_model
%     if C or A is observed in train_C/train_A, the posterior
%     post_pC/post_pA is set to 1 (or 0)
% post_pC: p(c|x,a)
% post_pA: p(a|x,c)

% pC_X i.e. p(C|X) is at least 0.01

assert(strcmpi(CA_model.type,type));

num = size(train_X, 1);

nclass = CA_model.nclass;
nafford = CA_model.nafford;
objmix = CA_model.objmix;

post_pC = zeros(num, nclass);
post_pA = train_A;


probC = CA_model.probC;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pX_C = zeros(num, nclass);
pC_X = zeros(num, nclass);

for i = 1:nclass
    pX_C(:,i) = gmmprob(objmix{i}, train_X);
end

pCX = pX_C .* (ones(num,1)*probC);

for k = 1:num
    this_pC_X = pCX(k,:) / sum(pCX(k,:));
    this_pC_X = this_pC_X + min_pc_x;
    pC_X(k,:) = this_pC_X / sum(this_pC_X);
end
%%%%%%%%  pC_X, pCX should not be changed here after !! %%%%%%%%%


switch type
    case 'CA_chain'
        affglm = [];
        probA_C = CA_model.probA_C;


        
        for i=1:num
            C = train_C(i);
            A = train_A(i,:);
            
            if C > 0  % C is observed
                post_pC(i,C) = 1;
                
                for j=1:nafford
                    if A(1,j) == -1 % otherwise it should be 0 or 1
                        A(1,j) = probA_C(C,j);
                    end
                end
            else % C is unobserved ( C = -1 )
                % not necessary, but put here for clarity
                this_pC_X = pC_X(i,:);
                
                for j=1:nafford
                    if A(1,j) == 0
                        this_pC_X = this_pC_X .* (1 - probA_C(:,j)');
                    elseif A(1,j) == 1
                        this_pC_X = this_pC_X .* probA_C(:,j)';
                    end
                end
                % before this step, this_pC_X contains p(A,C|X)
                % we calculate posterior by p(C|X,A) \prop p(A,C|X)
                post_pC(i,:) = this_pC_X / sum(this_pC_X);
                
                % now calculate the probability of missing A
                for j=1:nafford
                    if A(1,j) == -1
                        % this is an inner product
                        % which involves marginalize over C
                        A(1,j) = post_pC(i,:)*probA_C(:,j);
                    end
                end
            end
            post_pA(i,:)=A;
        end
        
        
    case 'CA_full'
        %%%%%%%%%%%%%%% (1) same as CA_con_LL %%%%%%%%%%%%%%
        probA_C = [];
        affglm = CA_model.affglm;

        % glm_pA_CX stores the matrix of p(A_k|C,X)
        % each element stores for fixed A_k and C
        glm_pA_CX = cell(nclass, nafford);
        glm_mask = zeros(nclass, nafford);

        for c = 1:nclass
            for j = 1:nafford
                glmtmp = affglm{c,j};
                if isstruct(glmtmp)
                    tmpfwd = glmfwd(glmtmp, train_X);
                    glm_pA_CX{c,j} = tmpfwd;
                    % mark it as special, where classifier is involved
                    glm_mask(c,j) = 1;
                elseif isscalar(glmtmp) && glmtmp >=0 && glmtmp<=1
                    glm_pA_CX{c,j} = glmtmp;
                    % glm_mask(c,j) = 0 by default
                else
                    error(['Unknown glm type for: c,j=' num2str([c,j])]);
                end
            end
        end
        %%%%%%%%%%%%%%%  (1) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        for i=1:num
            %%%%%%%%%%%%%% (2) same as CA_con_LL %%%%%%%%%%%%%%
            C = train_C(i);
            A = train_A(i,:);

            % p(A |C,X)
            pa_cx = zeros(nclass,nafford);
            for c=1:nclass
                for j=1:nafford
                    % special case, use classifier response
                    if glm_mask(c,j) == 1
                        pa_cx(c,j) = glm_pA_CX{c,j}(i);
                    else % mask(i,j) == 0
                        pa_cx(c,j) = glm_pA_CX{c,j};
                    end
                end
            end
            
            %%%%%%%%%%%%%%%  (2) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            if C > 0  % C is observed
                post_pC(i,C) = 1;

                for j=1:nafford
                    if A(1,j) == -1 % otherwise it should be 0 or 1
                        A(1,j) = pa_cx(C,j);
                    end
                end
            else % c is unobserved ( C = -1 )
                % not necessary, but put here for clarity
                this_pC_X = pC_X(i,:);
              
                for j=1:nafford
                    if A(1,j) == 0
                        this_pC_X = this_pC_X .* (1 - pa_cx(:,j)');
                    elseif A(1,j) == 1
                        this_pC_X = this_pC_X .* pa_cx(:,j)';
                    end
                end
                % before this step, this_pC_X contains p(A,C|X)
                % we calculate posterior by p(C|X,A) \prop p(A,C|X)
                post_pC(i,:) = this_pC_X / sum(this_pC_X);

                % now calculate the probability of missing A
                for j=1:nafford
                    if A(1,j) == -1
                        % this is an inner product
                        % which involves marginalize over C
                        A(1,j) = post_pC(i,:)*pa_cx(:,j);
                    end
                end
            end
            post_pA(i,:)=A;
        end
        % ppc = post_pC; ppa = post_pA; ppglm = glm_pA_CX;
        % save tmp2 ppc ppa ppglm
    otherwise
        error(['Unknown CA model type ', type]);
end