function [X, A, B, Aeq, Beq, LB, UB, paracount] = DP_get_para(DPgmm, train_X)
% [X, A, B, Aeq, Beq, LB, UB, paracount] = DP_get_para(DPgmm, train_X)
% DP: get the parameters and bounds for optimization from DPgmm

ratio = DPgmm.ratio;
posmix = DPgmm.posmix;
negmix = DPgmm.negmix;

dim = posmix.nin;

minX = min(train_X);
maxX = max(train_X);

minsig = ones(1,dim)*1e-6;
maxsig = ones(1,dim)*100;

nmix1 = posmix.ncentres;
nmix2 = negmix.ncentres;

num = dim*2*(nmix1+nmix2);
num = num + 1; % ratio
num = num + nmix1 -1; % priors for pos
num = num + nmix2 -1; % priors for neg
% disp(num); % real number of parameters

% dummy, for counting number of mixture components and dimension
num = num + 3;
paracount = num;

X = zeros(1, paracount);
LB = X;
UB = X;
Aeq = [];
Beq = [];

X(1) = dim;
X(2) = nmix1;
X(3) = nmix2;
LB(1,1:3)=-ones(1,3)*1e100;
UB(1,1:3)=ones(1,3)*1e100;

num=4;
X(num) = ratio;
LB(num) = 0.1;
UB(num) = 0.9;

A = zeros(2, paracount);
B = 0.95 * ones(2,1);

len = nmix1 - 1;
ind = num+1:num+len;
X(1,ind) = posmix.priors(1,1:end-1);
LB(1,ind) = ones(1,len) * 0.1;
UB(1,ind) = ones(1,len) * 0.9;
A(1,ind) = ones(1,len);
num = num + len;

len = nmix2 - 1;
ind = num+1:num+len;
X(1,ind) = negmix.priors(1,1:end-1);
LB(1,ind) = ones(1,len) * 0.1;
UB(1,ind) = ones(1,len) * 0.9;
A(2,ind) = ones(1,len);
num = num + len;

for i=1:nmix1
    ind = num+1:num+dim;
    X(1,ind) = posmix.centres(i,:);
    LB(1,ind) = minX;
    UB(1,ind) = maxX;
    num = num+dim;
    
    ind = num+1:num+dim;
    X(1,ind) = posmix.covars(i,:);
    LB(1,ind) = minsig;
    UB(1,ind) = maxsig;
    num = num+dim;
end

for i=1:nmix2
    ind = num+1:num+dim;
    X(1,ind) = negmix.centres(i,:);
    LB(1,ind) = minX;
    UB(1,ind) = maxX;
    num = num+dim;
    
    ind = num+1:num+dim;
    X(1,ind) = negmix.covars(i,:);
    LB(1,ind) = minsig;
    UB(1,ind) = maxsig; 
    num = num+dim;
end
assert(num == paracount);

