function [DPgmm_array, trainerr, testerr]  = DP_learn_GMM( ...
    train_X, train_A, test_X, test_A, aff, train_weight, ...
    nmix1, nmix2, method, max_DP_iter)
% DP: direct perception with Gaussian Mixture Model

% clear all; clc; load train_data
% nmix1 = 2;
% nmix2 = 2;
% max_DP_iter = 20;
% train_weight = [];
% aff = 2;
% method = 'generative';
% method = 'discriminative';

train_A = train_A(:,aff);
test_A = test_A(:,aff);

if isempty(train_weight)
    trainweight = 0;
else
    trainweight = 1;
    assert(any(train_weight>0));
end

DPgmm_array = cell(1,max_DP_iter);

dim = size(train_X, 2);

% given train_X, train_A
gmmiter = 20;
gmmeps = 1e-6;

train_A = train_A(:,1);
ind = find(train_A>-1);

train_X = train_X(ind,:);
train_A = train_A(ind,:);

numtrain = length(train_A);
numpos = sum(train_A == 1);
numneg = sum(train_A == 0);

posind = find(train_A == 1);
posdata = train_X(posind,:);
negind = find(train_A == 0);
negdata = train_X(negind,:);

% gmm initialization options
initoptions = foptions;
initoptions(14) = 5;
% gmm main loop options
gmmoptions = zeros(1, 18);
gmmoptions(14) = gmmiter;
gmmoptions(5) = 1;
gmmoptions(3) = gmmeps;

% nmixture components for both positive and negative
posmix = gmm(dim, nmix1, 'diag');
% intialization
posmix = gmminit(posmix, posdata, initoptions);

negmix = gmm(dim, nmix2, 'diag');
% intialization
negmix = gmminit(negmix, negdata, initoptions);

if trainweight == 0
    ratio = numpos / numneg;
    % real em for gmm learning
    posmix = gmmem(posmix, posdata, gmmoptions);
    negmix = gmmem(negmix, negdata, gmmoptions);
else
    % multiply a constant should not matter
    posweight = train_weight(posind,:) *100;
    negweight = train_weight(negind,:) *100;
    ratio = sum(posweight) / (sum(posweight)+sum(negweight));
    posmix = gmmem(posmix, posdata, posweight, gmmoptions);
    negmix = gmmem(negmix, negdata, negweight, gmmoptions);
end
[numpos numneg ratio]

DPgmm = struct('posmix',posmix,'negmix',negmix,'ratio',ratio);
DPgmm_array{1} = DPgmm;

switch lower(method)
    case {'generative','gen'}
        return;
        
    case {'discriminative','dis'}
        % discriminatively optimize the CLL function
        
        [X0, A, B, Aeq, Beq, LB, UB, paracount] = DP_get_para(DPgmm, train_X);

        alldata = struct('DPgmm',DPgmm, 'A', train_A, 'X', train_X, ...
            'fullX', X0, 'LB', LB, 'UB', UB, 'index', 1:paracount);

        opt_maxiter = 3;
        options = optimset('Display','off', ... % off/iter/notify/final
            'FunValCheck','on',...
            'TolFun',1e-6,'MaxIter',opt_maxiter,'MaxFunEvals',1e6, ...
            'LargeScale','off');

        iter = 1;
        prob = DP_prob(DPgmm, train_X);
        prob = (prob>=0.5);
        trainerr(1,iter) = sum(train_A~=prob) / length(prob);
        prob = DP_prob(DPgmm, test_X);
        prob = (prob>=0.5);
        testerr(1,iter) = sum(test_A(:,1)~=prob) / length(prob);
        
        DPindex = DPgmm;
        Xindex = 1:length(X0);
        Xindex(1,1:3)=X0(1,1:3);
        DPindex = DP_put_para(DPindex,Xindex);
        
        nmix = [nmix1 nmix2];
        assert(all(nmix==X0(2:3)));
        len = 1+(nmix1-1)+(nmix2-1); %ratio, priors
        baseindex = 4:4+len-1;
        assert(baseindex(end)==DPindex.posmix.centres(1,1)-1);
        % X0(baseindex)
        % count = zeros(size(X0));
        for iter = 2:max_DP_iter
            for i = 1:2 % pos, neg
                if i == 1 % pos
                    tmpmix = DPindex.posmix;
                    num_tmpmix = nmix1;
                else % neg
                    tmpmix = DPindex.negmix;
                    num_tmpmix = nmix2;
                end

                for j = 1:num_tmpmix
                    index = [baseindex ...
                        tmpmix.centres(j,:) ...
                        tmpmix.covars(j,:)];
                    %% count how many times this dim has been
                    %% searched in optimization
                    % count(index)=count(index)+1;

                    % update the index for searching
                    alldata.index = index;

                    pX0 = alldata.fullX(index);
                    pA = A(:,index);
                    pB = B;
                    pLB = LB(index);
                    pUB = UB(index);

                    [pX1,FVAL,EXITFLAG,OUTPUT,LAMBDA,GRAD,HESSIAN] = ...
                        fmincon(@(X)DP_fun_CLL_subset(X,alldata), ...
                        pX0,pA,pB,Aeq,Beq,pLB,pUB,[],options);

                    % update with the new parameter subset
                    alldata.fullX(index) = pX1;

                    % disp([iter,i,j,FVAL]);
                end
            end
            % disp([iter FVAL]);
            % assert(sum(count)==(1+nmix1+nmix2-2) ...
            %    *(nmix1+nmix2)+(nmix1+nmix2)*dim*2);
            newDPgmm = DP_put_para(DPgmm, alldata.fullX);
            
            DPgmm_array{iter}=newDPgmm;

            prob = DP_prob(newDPgmm, train_X);
            prob = (prob>=0.5);
            trainerr(1,iter) = sum(train_A~=prob) / length(prob);
            prob = DP_prob(newDPgmm, test_X);
            prob = (prob>=0.5);
            testerr(1,iter) = sum(test_A(:,1)~=prob) / length(prob);
        end
        
        
    otherwise
        error(['Error unknown training method:' method]);
        
end

% figure;
% plot(trainerr,'LineWidth',2);
% hold on;
% plot(testerr,'r-','LineWidth',2);