% batch learning with subset
%
%

clear all;

total_p = 5;
CA_chain_maxiter = 20;
CATENFORCE = 1

chain_trainErr = zeros(total_p,CA_chain_maxiter,6);
chain_testErr = zeros(total_p,CA_chain_maxiter,6);

all_CA = cell(1,total_p);

for p = 1:total_p
    script_learn_CA_chain_gen_EM;

    
    % This needs to be here to overwrite the save parameter in chain_gen_EM
    CA_chain_maxiter = 20;
    
    
    clear GLOBAL_X GLOBAL_MINLLF GLOBAL_ID;
    global GLOBAL_X GLOBAL_MINLLF GLOBAL_ID;
    
    GLOBAL_ID = 0;

    % start from the most recent model: CA_chain
    [fullX0, A, B, Aeq, Beq, LB, UB] = get_CA_para(CA_chain, train_X);

    alldata=struct('CAmodel', CA_chain, ...
        'A', train_A, 'C', train_C, 'X', train_X, ... % training data
        'LB', LB, 'UB', UB, 'fullX', fullX0, 'ind', 1:454, ... % full parameter set
        'condtype', 'pC_X',  'post_pC',post_pC);

    minLLF = fun_CA_weight_cond_LL_subset(fullX0,alldata)


    %%%%%%%%%%%%% Real optimization procedure %%%%%%%%%%%%%%%%%


    dispstar();
    disp(['CondLL type to maximize = ' alldata.condtype]);

    CA_chain_subset_array = cell(1,CA_chain_maxiter);

    for iter = 1:CA_chain_maxiter
        displine(); disp(['Iter = ' num2str(iter)]);
        
        opt_maxiter = 5;
        options = optimset('Display','off', ... % off/iter/notify/final
            'FunValCheck','on',...
            'TolFun',1e-6,'MaxIter',opt_maxiter,'MaxFunEvals',1e6, ...
            'LargeScale','off');

        for obj = 1:7
            if obj <8
            % t = 28 + (obj-1)*61;
            % index = [1:6, t:t+60];
            % 2 mixutre -> tmp=61
            % 1 mixutre -> tmp=30
            tmp = nmixture*dim*2+nmixture-1;
            t = 28 + (obj-1)*tmp;
            index = [1:6, t:t+tmp-1];

            end

            % partial vector for optimization
            alldata.ind = index;
            % the most recent fullX is in alldata.fullX
            pX0 = alldata.fullX(index);
            pA = A(index);
            pB = B;
            pLB = LB(index);
            pUB = UB(index);

            [pX1,FVAL,EXITFLAG,OUTPUT,LAMBDA,GRAD,HESSIAN] = ...
                fmincon(@(X)fun_CA_weight_cond_LL_subset(X,alldata), ...
                pX0,pA,pB,Aeq,Beq,pLB,pUB,[],options);

            % display purpose
            if obj <= nclass
                displine(); disp(['Category ' num2str(obj) '     CLL=' num2str(FVAL)]);
            end

            % put the partial optimized X into the full X
            newfullX=alldata.fullX;
            newfullX(index)=pX1;
            alldata.fullX=newfullX;
        end

        CA_chain = put_CA_para(CA_chain, alldata.fullX);
        
        array_cll(iter) = FVAL;
        
        disp(['iter ' num2str(iter) ': ' num2str(array_cll(iter))]);

        if CATENFORCE == 1
            [post_pC, post_pA, post_pCX] = CA_posterior(CA_chain, 'CA_chain', ...
                train_X, train_C, train_A, MIN_POST_C);
        else
            empty_C = - ones(size(train_C));
            [post_pC, post_pA, post_pCX] = CA_posterior(CA_chain, 'CA_chain', ...
                train_X, empty_C, train_A, MIN_POST_C);
        end

        CA_chain = learn_CA_chain_para(CA_chain, ...
            post_pC, post_pA, CA_para_epsilon);

        CA_chain_subset_array{iter} = CA_chain;

    end
    
    all_CA{p}=CA_chain_subset_array;
    
    for k = 1:CA_chain_maxiter
        CA_chain = CA_chain_subset_array{k};
        chain_trainErr(p,k,:) = get_CA_err(CA_chain, train_X, train_C, train_A, 0.001);
        chain_testErr(p,k,:) = get_CA_err(CA_chain, test_X, test_C, test_A, 0.001);
        ll = CA_joint_LL(CA_chain, 'CA_chain', train_X, train_C, train_A, 'pXA');
        trainLL(p,k) = sum(ll);
        ll = CA_joint_LL(CA_chain, 'CA_chain', test_X, test_C, test_A, 'pXA');
        testLL(p,k) = sum(ll);
        [trainCLL(p,k), tmp] = CA_cond_LL(CA_chain, 'CA_chain', ...
            train_X, train_C, train_A, 'pA_X_sep');
        [testCLL(p,k), tmp] = CA_cond_LL(CA_chain, 'CA_chain', ...
            test_X, test_C, test_A, 'pA_X_sep');
        [trainCLL_grp(p,k), tmp] = CA_cond_LL(CA_chain, 'CA_chain', ...
            train_X, train_C, train_A, 'pA_X_group');
        [testCLL_grp(p,k), tmp] = CA_cond_LL(CA_chain, 'CA_chain', ...
            test_X, test_C, test_A, 'pA_X_group');
    end    
end

% dispstar(); disp('Learning finished');
% disp('Plot the error and conditional probabilities');
% script_plot_error
if CATENFORCE ==1 
    save batch_chain_subset_EM
else
    save batch_chain_subset_EM_noenforce
end
modeltype = 'CA_chain';
batch_plot_errorbar;