function [train_data, test_data] = generate_traintest_data( ...
    affdata, projdata1, projdata2, dim1, dim2, ...
    numtrain, numtest, ...
    ratio)
% CA: random sample to generate the training/testing data
%	numtrain/numtest -> number of data
%	affdata -> contains affordance value of each individual original data
%	projdata1/projdata2 -> projected data after PCA
%	dim1/dim2 -> dimension after PCA
% Note: ratio -> since projdata2's value >> projdata1, we multiply that by a ratio

assert(size(projdata1{1},2)>=dim1);
assert(size(projdata2{1},2)>=dim2);

nclass = length(projdata1);
numtotal = numtrain + numtest;

train_data=struct('X',[],'C',[], 'A', [], 'std', []);
test_data=struct('X',[],'C',[], 'A', [], 'std', []);

train_X=[]; test_X=[];
train_C=[]; test_C=[];
train_A=[]; test_A=[];
train_std=[]; test_std=[];

for i = 1:nclass
    count=size(projdata1{i},1); % num data points

    index = rand_sample(1:count,numtotal,0); % do not sort the index
    ind = index(1:numtrain);
    
    oneX = [projdata1{i}(ind,1:dim1) projdata2{i}(ind,1:dim2)*ratio];
    one_std = std(oneX);
    
    train_X = [train_X; oneX];
    train_C = [train_C; i*ones(numtrain,1)];
    train_A = [train_A; affdata{i}(ind,:)];
    train_std = [train_std; one_std];
    
    ind = index(numtrain+1:end);
    oneX = [projdata1{i}(ind,1:dim1) projdata2{i}(ind,1:dim2)*ratio];
    one_std = std(oneX);
    
    test_X = [test_X; oneX];
    test_C = [test_C; i*ones(numtest,1)];
    test_A = [test_A; affdata{i}(ind,:)];
    test_std = [test_std; one_std];
end

train_data.X = train_X;
train_data.C = train_C;
train_data.A = train_A;
train_data.std = train_std;

test_data.X = test_X;
test_data.C = test_C;
test_data.A = test_A;
test_data.std = test_std;