function CA_chain = learn_CA_chain_para(CA_chain, pC, pA, epsilon)
% CA: Learn P(C=i) & P(A_j=1 | C=i) parameters of CA-chain model
%
% Inputs: (posterior of the data)
%   CA_chain -> model structure
%   pC -> posterior p(C|X,A)
%   pA -> posterior p(A|X,C)
%
% Outputs: (model parameters)
%   CA_chain.probC(i) -> P(C=i)
%   CA_chain.probA_C(i,j) -> P(A_j=1 | C=i)

numobj = size(pC,2);
numaff = size(pA,2);
num = size(pC,1);

mask = (pC==-1);

if sum(mask(:)) == 0 % no data is missing
    probC = sum(pC);
    % make sure pC add up together is close to num
    assert(approxeq(num, sum(probC)));
else
    tmp = sum(mask');
    % the rows that C is observed ( ~= -1)
    ind = find(tmp == 0); 
    % if C is missing, each row has numobj # of -1
    % the sum should be numobj
    ind2 = find(tmp == numobj); 
    
    assert(length(ind)+length(ind2) == num);
    
    probC = sum(pC(ind',:));
end

%%% Add a small value to avoid singulairty %%%
probC = probC + epsilon;

probC = probC / sum(probC);

for i = 1:numobj
    for j = 1:numaff
        pc = pC(:,i);
        pa = pA(:,j);
        
        % filter out the unknowns
        ind = (pc>-1) & (pa>-1);
        
        pc=pc(ind,1);
        pa=pa(ind,1);
        
        pac = sum(pc.*pa);
        probA_C(i,j) = pac / sum(pc);
    end
end

%%% Add a small value to avoid singulairty %%%
probA_C = max(probA_C, epsilon);
probA_C = min(probA_C, (1-epsilon));

CA_chain.probC = probC;
CA_chain.probA_C = probA_C;

