function [newmix, ll] = learn_category_mixture(gmmdata, weight, ...
    nmixture, maxiter, epsilon, prior_cov, ...
    cov_type, model, mix)
% CA: learn a GMM per object class
%	model -> choosing from three methods:
%		'KPM': Kevin Murphy's implementation
%		'Netlab': Netlab implementation (use this if possible)
%		'Weighted': Weighted version based on Netlab's implementation
%	cov_type -> covariance type
%		'diag': recommended
%		'full': not recommended (using KPM is better, Weighted is based on Netlab thus not good)
% RETURN:
%       it returns LLF now, instead of -LLF, therefore the larger the
%       better

dim = size(gmmdata, 2);

%% Requried by Netlab's implementation (same for weighted) %%
options = zeros(1, 18);
% options(1) = 1; % Prints out error values.
options(14) = maxiter; % Max. Number of iterations.
options(5) = 1; % check covariance to be non-singular
options(3) = epsilon;

switch model
    case 'KPM'
        assert(isempty(weight));
        if options(1); disp(' '); disp('Using KPM''s gmmem2() implementation'); end;
        %% KP Murphy's implementation -- better for 'full' cov_type %%
        [newmix, numiter, ll] = gmmem2(mix, gmmdata, 'max_iter', maxiter, ...
            'll_thresh', epsilon, 'prior_cov', prior_cov, 'verbose', 0);

    case 'Netlab'
        assert(isempty(weight));
        if options(1); disp(' '); disp('Using Netlab''s gmmem() implementation'); end;
        %% Netlab's implementation %%
        [newmix, options, errlog] = gmmem(mix, gmmdata, options);
        ll=options(8);  % options(8) returns LLF now (modified)
        
    case 'Weighted'
        assert(~isempty(weight));
        if options(1); disp(' '); disp('Using Weighted gmmem_weight() modification'); end;
        %% Modified implementation with weighted data %%
        [newmix, options, errlog] = gmmem_weight(mix, gmmdata, weight, options);
        ll=options(8);  % options(8) returns LLF (modified)

    otherwise
        error(['Unknown model type ', model]);
end
