function [new_CA_chain, trainerr, trainnumA, testerr, testnumA] = new_learn_affordance(CA_chain, ...
    train_X, train_A, test_X, test_A, method)

% [new_CA_chain, trainerr, trainnumA, testerr, testnumA] = new_learn_affordance(CA, train_X(1:100:end,:), train_A(1:100:end,4), test_X, test_A(:,4), 'full');

% just set it to CA_chain type
% probA_C is not useful
CA_chain.type = 'CA_chain';

MIN_POST_C  = 0.001;
CA_para_epsilon = 0.05;

num = size(train_X,1);
empty_C = - ones(num,1);

nafford = CA_chain.nafford;
nclass = CA_chain.nclass;
empty_A = - ones(num,nafford);

[post_pC, post_pA, post_pC_X] = CA_posterior(CA_chain, 'CA_chain', ...
    train_X, empty_C, empty_A, MIN_POST_C);

% post_pC and post_pC_X should equal with empty_C and empty_A input
% norm(post_pC - post_pC_X, 2)

train_post_pC = post_pC_X;

posindex = find(train_A == 1);
negindex = find(train_A == 0);

pos_pC_X = post_pC_X(posindex,:);
neg_pC_X = post_pC_X(negindex,:);

pos_pC = sum(pos_pC_X);
neg_pC = sum(neg_pC_X);
pC = pos_pC + neg_pC;

pC = max(pC, 0.01);
pA_C = pos_pC ./ pC;

pA_C = max(pA_C, CA_para_epsilon);
pA_C = min(pA_C, 1 - CA_para_epsilon);

new_CA_chain = CA_chain;
probA_C = CA_chain.probA_C;

new_CA_chain.nafford = nafford + 1;
new_probA_C = [probA_C pA_C'];
new_CA_chain.probA_C = new_probA_C;

empty_A = - ones(num,nafford+1);
[post_pC, post_pA, post_pC_X] = CA_posterior(new_CA_chain, 'CA_chain', ...
    train_X, empty_C, empty_A, MIN_POST_C);

pA = (post_pA >= 0.5);
pA = pA(:,nafford+1);

numerr = sum(pA~=train_A & train_A>-1);
trainnumA = sum(train_A>-1);
trainerr = numerr / trainnumA;

num = size(test_X,1);
empty_C = - ones(num,1);
empty_A = - ones(num,nafford+1);

[post_pC, post_pA, post_pC_X] = CA_posterior(new_CA_chain, 'CA_chain', ...
    test_X, empty_C, empty_A, MIN_POST_C);

pA = (post_pA >= 0.5);
pA = pA(:,nafford+1);

numerr = sum(pA~=test_A & test_A>-1);
testnumA = sum(test_A>-1);
testerr = numerr / testnumA;

nafford = nafford + 1;

dim = 15;

disp([trainnumA, testnumA]);
disp('chain: trainerr, testerr');
disp([trainerr, testerr]);

switch method
    case 'chain'
        return;
    case 'full'
        CA_full = new_CA_chain;
        CA_full.type = 'CA_full';
        CA_full.affglm = [];
        CA_full.nafford = 1; %only 1 affordance
        CA_full.probA_C = pA_C'; %from chain model
        
        affglm = cell(nclass,1);

        for obj = 1:nclass
            prob_pa_c = CA_full.probA_C(obj);
            
            glmdata = train_X;
            glmtargets = train_A;

            glmweight = train_post_pC(:,obj);

            ind_weight = (glmweight > 0.1);
            ind_aff = (glmtargets > -1);
            ind = ind_weight & ind_aff;

            glmdata = glmdata(ind,:);
            glmtargets = glmtargets(ind,:);
            glmweight = glmweight(ind,:);

            pos = (glmtargets==1);
            neg = (glmtargets==0);
            npos = sum(pos);
            nneg = sum(neg);
                
            % [npos nneg prob_pa_c]
            
            %%% might need to change this
            PA_C_THRESH = 0.05;
            
            if prob_pa_c==PA_C_THRESH || prob_pa_c== 1- PA_C_THRESH
                % almost sufficiency holds
                CA_full.affglm{obj,1} = prob_pa_c;

            elseif npos+nneg < 10
                % too few training data
                CA_full.affglm{obj,1} = prob_pa_c;
            
            else
                % learn the linear classifier
                glmnet = glm(dim, 1, 'logistic');
                        
                options = foptions;
                options(2) = 1e-2;
                options(14) = 5; % num iteration

                [glmnet, opt] = glmtrain_weight(glmnet, options, ...
                    glmdata, glmweight, glmtargets);

                CA_full.affglm{obj,1} = glmnet;
            end
        end
        train_C = ones(size(train_A));
        test_C = ones(size(test_A));
        min_post_c = 0.001;
        
        [trainerr, post_pC] = get_CA_err(CA_full, ...
            train_X, train_C, train_A, min_post_c);

        [testerr, post_pC] = get_CA_err(CA_full, ...
            test_X, test_C, test_A, min_post_c);
        
        new_CA_chain = CA_full;
        trainerr = trainerr(1);
        testerr = testerr(1);

        disp('full: trainerr, testerr');
        disp([trainerr, testerr]);
    case 'KNN'

end
