function [projdata, avg, sigma, projV, V, d] = pca_projection(data, projdim)
% CA: called by pca_experiment()
% this the actual code to do pca projection to a low dimensional space

% for each matrix in data, we get the average
numobj = length(data);
for i = 1:numobj
    objcount(i)=size(data{i},1);
    objavg(i,:) = mean(data{i});
end

% calculate the total average of all data points
dim = size(objavg,2);
avg=zeros(1,dim);
for i=1:numobj
    avg=avg+objcount(i)*objavg(i,:);
end
count=sum(objcount);
avg=avg/count;

% for colorhistogram, calculate the covariance matrix
sigma = zeros(dim,dim);
for i=1:numobj
    num=objcount(i);
    newdata = data{i} - ones(num,1)*avg;
    sigma = sigma + newdata'*newdata;
end
sigma = sigma / (count-1);

% 1. before PCA
% for all the data, the total count, mean, covariances are:
% count, avg, sigma
% the last dimensions have the most significants

[V,d]=eig(sigma);
projV = V(:,end-projdim+1:end);

% this makes sure that the first dimension is the largest
projV = fliplr(projV);

projsigma = zeros(projdim,projdim);
for i=1:numobj
     num=objcount(i);
     newdata = data{i} - ones(num,1)*avg;
     newdata = newdata*projV;
     projsigma = projsigma + newdata'*newdata;
end
projsigma = projsigma / (count-1);

% 2. using PCA
% newdata = (original-average)*projection
% the projection matrix is projV
% the new covariance is projsigma
projdata=cell(1,numobj);
for i =1:numobj
    num=objcount(i);
    newdata = data{i}-ones(num,1)*avg;
    newdata = newdata*projV;
    projdata{i}=newdata;
end


