% CA: script to learn a CA_chain model Discriminatively
% clear all; clc; close all; load init_mixture.mat;
% 
% % call the EM script
% script_learn_CA_chain_gen_EM
% 
% save CA_chain_gen;
% return;

clear all;

script_learn_CA_chain_gen_EM;
% load CA_chain_gen.mat;

CA_chain_maxiter = 20;

global GLOBAL_X GLOBAL_MINLLF GLOBAL_ID;
GLOBAL_ID = 0;

% start from the most recent model: CA_chain
[fullX0, A, B, Aeq, Beq, LB, UB] = get_CA_para(CA_chain, train_X);

alldata=struct('CAmodel', CA_chain, ...
    'A', train_A, 'C', train_C, 'X', train_X, ... % training data
    'LB', LB, 'UB', UB, 'fullX', fullX0, 'ind', 1:length(fullX0), ... % full parameter set
    'condtype', 'pA_X_group'); %%!!!! cond prob type


minLLF = fun_CA_cond_LL_subset(fullX0,alldata);

%%%%%%%%%%%%% Real optimization procedure %%%%%%%%%%%%%%%%%


dispstar();
disp(['CondLL type to maximize = ' alldata.condtype]);

CA_chain_subset_array = cell(1,CA_chain_maxiter);

for iter = 1:CA_chain_maxiter
    displine(); disp(['Iter = ' num2str(iter)]);
    
    opt_maxiter = 5;
    options = optimset('Display','off', ... % off/iter/notify/final
        'FunValCheck','on',...
        'TolFun',1e-6,'MaxIter',opt_maxiter,'MaxFunEvals',1e6, ...
        'LargeScale','off');
    
    for obj = 1:8
        if obj <8
            % 2 mixutre -> tmp=61
            % 1 mixutre -> tmp=30
            tmp = nmixture*dim*2+nmixture-1;
            t = 28 + (obj-1)*tmp;
            index = [1:6, t:t+tmp-1];
        else % there is no object 8, it's just the p(A|C) parameter
            index = [7:27];
        end

        % partial vector for optimization
        alldata.ind = index;
        % the most recent fullX is in alldata.fullX
        pX0 = alldata.fullX(index);
        pA = A(index);
        pB = B;
        pLB = LB(index);
        pUB = UB(index);

        [pX1,FVAL,EXITFLAG,OUTPUT,LAMBDA,GRAD,HESSIAN] = ...
            fmincon(@(X)fun_CA_cond_LL_subset(X,alldata), ...
            pX0,pA,pB,Aeq,Beq,pLB,pUB,[],options);

        % display purpose
        if obj <= nclass
            displine(); disp(['Category ' num2str(obj) '     CLL=' num2str(FVAL)]);
        else
            displine(); disp(['P(A|C)     -CLL=' num2str(FVAL)]);
        end
        
        % put the partial optimized X into the full X
        newfullX=alldata.fullX;
        newfullX(index)=pX1;
        alldata.fullX=newfullX;

    end
    
    CA_chain = put_CA_para(CA_chain, alldata.fullX);
    
    CA_chain_subset_array{iter} = CA_chain;

    % displine(); disp('Calculate the error and conditional probabilities');
    % script_calc_error
end

dispstar();
disp('Learning finished');

total_p = 1;
chain_trainErr = zeros(total_p, CA_chain_maxiter, 6);
chain_testErr = chain_trainErr;

for k = 1:CA_chain_maxiter
    CA_chain = CA_chain_array_subset{k};
    chain_trainErr(p,k,:) = get_CA_err(CA_chain, train_X, train_C, train_A, 0.001);
    chain_testErr(p,k,:) = get_CA_err(CA_chain, test_X, test_C, test_A, 0.001);
    ll = CA_joint_LL(CA_chain, 'CA_chain', train_X, train_C, train_A, 'pXA');
    trainLL(p,k) = sum(ll);
    ll = CA_joint_LL(CA_chain, 'CA_chain', test_X, test_C, test_A, 'pXA');
    testLL(p,k) = sum(ll);
    [trainCLL(p,k), tmp] = CA_cond_LL(CA_chain, 'CA_chain', ...
        train_X, train_C, train_A, 'pA_X_sep');
    [testCLL(p,k), tmp] = CA_cond_LL(CA_chain, 'CA_chain', ...
        test_X, test_C, test_A, 'pA_X_sep');
    [trainCLL_grp(p,k), tmp] = CA_cond_LL(CA_chain, 'CA_chain', ...
        train_X, train_C, train_A, 'pA_X_group');
    [testCLL_grp(p,k), tmp] = CA_cond_LL(CA_chain, 'CA_chain', ...
        test_X, test_C, test_A, 'pA_X_group');
end
% figure(1000); plot(array_pxa_ll);

modeltype = 'CA_chain';
batch_plot_errorbar;
% disp('Plot the error and conditional probabilities');
% script_plot_error
