% CA: script to learn a CA_chain model Generatively through EM method
load train_data

CA_chain_maxiter = 20;
CA_chain_array = cell(1,CA_chain_maxiter);

MIN_POST_C  = 0.01;
CA_para_epsilon = 0.05;

nclass = 7;
nmixture = 2;
gmm_maxiter = 10;
gmm_epsilon = 1e-6;
gmm_prior_cov = eye(dim)*1e-6; % min covariance matrix
objmix = cell(1,nclass);
for obj = 1:nclass
    ind = find(train_C == obj);
    gmmdata = train_X(ind,:);

    oldmix = gmm(dim, nmixture, 'diag');
    options = foptions; options(14) = 5; % 5 iter of k-means to init
    oldmix = gmminit(oldmix, gmmdata, options);

    [objmix{obj}, objll(obj)] = learn_category_mixture(gmmdata, [], ...
        nmixture, gmm_maxiter, gmm_epsilon, gmm_prior_cov, ...
        'diag', 'Netlab', oldmix); % cov_type, model(KPM/Netlab/Weighted)
end

displine();
disp('Initialize the CA_chain model');
CA_chain = init_CA_model('CA_chain');
CA_chain.nclass = nclass;
CA_chain.nmixture = nmixture;
CA_chain.nafford = nafford;
CA_chain.objmix = objmix;
CA_chain.ndim = dim;

disp(['Min posterior of C is : ' num2str(MIN_POST_C)]);

disp('Construct initial posterior: pC, pA');
num = size(train_X,1);
pC = zeros(num, nclass);
pA = train_A;
for i=1:num
    c = train_C(i);
    if c == -1 % if C is not observed
        %% Or even give it a smaller value, implying smaller weight %%
        pC(i,:) = (-1) * ones(1,nclass);
    else
        pC(i,c) = 1; % others are 0
    end
end

disp('Calcualte initial prob para estimation: probC, probA_C');
CA_chain = learn_CA_chain_para(CA_chain, pC, pA, CA_para_epsilon);

disp('Calculate posterior from initial model');
[post_pC, post_pA] = CA_posterior(CA_chain, 'CA_chain', ...
    train_X, train_C, train_A, MIN_POST_C);


clear weight;

array_cll = zeros(1, CA_chain_maxiter);
for iter = 1 : CA_chain_maxiter
    % CA_chain.probC
    % CA_chain.probA_C
    % start from the most recent model: CA_chain
    [fullX0, A, B, Aeq, Beq, LB, UB] = get_CA_para(CA_chain, train_X);

    alldata=struct('CAmodel', CA_chain, ...
        'A', train_A, 'C', train_C, 'X', train_X, ... % training data
        'LB', LB, 'UB', UB, 'fullX', fullX0, 'ind', 1:454, ... % full parameter set
        'condtype', 'pC_X',  'post_pC',post_pC);
    %%!!!! cond prob type should be pC_X

    opt_maxiter = 5;
    options = optimset('Display','off','FunValCheck','on',...
        'TolFun',1e-6,'MaxIter',opt_maxiter,'MaxFunEvals',1e6, ...
        'LargeScale','off');

    minLLF = fun_CA_weight_cond_LL_subset(fullX0,alldata)

    
     for obj = 1:7
        if obj <8
            % t = 28 + (obj-1)*61;
            % index = [1:6, t:t+60];
            % 2 mixutre -> tmp=61
            % 1 mixutre -> tmp=30
            tmp = nmixture*dim*2+nmixture-1;
            t = 28 + (obj-1)*tmp;
            index = [1:6, t:t+tmp-1];
        end

        % partial vector for optimization
        alldata.ind = index;
        % the most recent fullX is in alldata.fullX
        pX0 = alldata.fullX(index);
        pA = A(index);
        pB = B;
        pLB = LB(index);
        pUB = UB(index);

        [pX1,FVAL,EXITFLAG,OUTPUT,LAMBDA,GRAD,HESSIAN] = ...
            fmincon(@(X)fun_CA_weight_cond_LL_subset(X,alldata), ...
            pX0,pA,pB,Aeq,Beq,pLB,pUB,[],options);

        % display purpose
        if obj <= nclass
            displine(); disp(['Category ' num2str(obj) '     CLL=' num2str(FVAL)]);
        end

        % put the partial optimized X into the full X
        newfullX=alldata.fullX;
        newfullX(index)=pX1;
        alldata.fullX=newfullX;
    end   
    

    CA_chain = put_CA_para(CA_chain, alldata.fullX);
    
    array_cll(iter) = FVAL;
    
    disp(['iter ' num2str(iter) ': ' num2str(array_cll(iter))]);

    % estimate the posterior based on current model
    [post_pC, post_pA, post_pCX] = CA_posterior(CA_chain, 'CA_chain', ...
        train_X, train_C, train_A, MIN_POST_C);
    
    % (6.2) learn the probC & probA_C parameters
    CA_chain = learn_CA_chain_para(CA_chain, ...
        post_pC, post_pA, CA_para_epsilon);

    %%% store the learned model !!!
    CA_chain_array{iter} = CA_chain;
    
end

total_p = 1;
chain_trainErr = zeros(total_p, CA_chain_maxiter, 6);
chain_testErr = chain_trainErr;

for k = 1:CA_chain_maxiter
    CA_chain = CA_chain_array{k};
    chain_trainErr(p,k,:) = get_CA_err(CA_chain, train_X, train_C, train_A, 0.001);
    chain_testErr(p,k,:) = get_CA_err(CA_chain, test_X, test_C, test_A, 0.001);
    ll = CA_joint_LL(CA_chain, 'CA_chain', train_X, train_C, train_A, 'pXA');
    trainLL(p,k) = sum(ll);
    ll = CA_joint_LL(CA_chain, 'CA_chain', test_X, test_C, test_A, 'pXA');
    testLL(p,k) = sum(ll);
    [trainCLL(p,k), tmp] = CA_cond_LL(CA_chain, 'CA_chain', ...
        train_X, train_C, train_A, 'pA_X_sep');
    [testCLL(p,k), tmp] = CA_cond_LL(CA_chain, 'CA_chain', ...
        test_X, test_C, test_A, 'pA_X_sep');
    [trainCLL_grp(p,k), tmp] = CA_cond_LL(CA_chain, 'CA_chain', ...
        train_X, train_C, train_A, 'pA_X_group');
    [testCLL_grp(p,k), tmp] = CA_cond_LL(CA_chain, 'CA_chain', ...
        test_X, test_C, test_A, 'pA_X_group');
end
% figure(1000); plot(array_pxa_ll);

modeltype = 'CA_chain';
batch_plot_errorbar;
