% CA: script to learn a CA_chain model Generatively through EM method
load train_data

CA_chain_maxiter = 20;
CA_chain_array = cell(1,CA_chain_maxiter);

MIN_POST_C  = 0.01;
CA_para_epsilon = 0.05;

nclass = 7;
nmixture = 2;
gmm_maxiter = 10;
gmm_epsilon = 1e-6;
gmm_prior_cov = eye(dim)*1e-6; % min covariance matrix
objmix = cell(1,nclass);
for obj = 1:nclass
    ind = find(train_C == obj);
    gmmdata = train_X(ind,:);

    oldmix = gmm(dim, nmixture, 'diag');
    options = foptions; options(14) = 5; % 5 iter of k-means to init
    oldmix = gmminit(oldmix, gmmdata, options);

    [objmix{obj}, objll(obj)] = learn_category_mixture(gmmdata, [], ...
        nmixture, gmm_maxiter, gmm_epsilon, gmm_prior_cov, ...
        'diag', 'Netlab', oldmix); % cov_type, model(KPM/Netlab/Weighted)
end

displine();
disp('Initialize the CA_chain model');
CA_chain = init_CA_model('CA_chain');
CA_chain.nclass = nclass;
CA_chain.nmixture = nmixture;
CA_chain.nafford = nafford;
CA_chain.objmix = objmix;
CA_chain.ndim = dim;

disp(['Min posterior of C is : ' num2str(MIN_POST_C)]);


disp('Construct initial posterior: pC, pA');
num = size(train_X,1);
pC = zeros(num, nclass);
pA = train_A;
for i=1:num
    c = train_C(i);
    if c == -1 % if C is not observed
        %% Or even give it a smaller value, implying smaller weight %%
        pC(i,:) = (-1) * ones(1,nclass);
    else
        pC(i,c) = 1; % others are 0
    end
end

disp('Calcualte initial prob para estimation: probC, probA_C');
CA_chain = learn_CA_chain_para(CA_chain, pC, pA, CA_para_epsilon);

disp('Calculate posterior from initial model');
[post_pC, post_pA] = CA_posterior(CA_chain, 'CA_chain', ...
    train_X, train_C, train_A, MIN_POST_C);

% displine();
% disp('(6) EM on learning mixture & model parameters');

clear weight;

objll = zeros(CA_chain_maxiter, nclass);

objmix_prev = objmix;
for iter = 1 : CA_chain_maxiter
    % (6.1) learn weighted gmm model for categories
    for obj = 1:nclass
       
        % get training data for each class
        gmmdata = train_X;

        %%%% setting the weight for each data point %%%%
        % disp('setting the weight for each data point');
        gmmweight = post_pC(:,obj);
        
        % filter out if weight is too small
        ind = gmmweight > 0.05;
        gmmdata = gmmdata(ind,:);
        gmmweight = gmmweight(ind,:);
        
        oldmix =  objmix_prev{obj};

        [newmix, newll] = learn_category_mixture( ...
            gmmdata, gmmweight, ...
            nmixture, gmm_maxiter, gmm_epsilon, gmm_prior_cov, ...
            'diag', 'Weighted', ... % cov_type, model(KPM/Netlab/Weighted)
            oldmix);
        
%         for t = 1:3
%             oldmix = gmm(dim, nmixture, 'diag');
%             options = foptions;
%             options(14) = 5;	% 5 iter of k-means to init
%             oldmix = gmminit(oldmix, gmmdata, options);
%             [mix, ll] = learn_category_mixture( ...
%                 gmmdata, gmmweight, ...
%                 nmixture, gmm_maxiter, gmm_epsilon, gmm_prior_cov, ...
%                 'diag', 'Weighted', ... % cov_type, model(KPM/Netlab/Weighted)
%                 oldmix);
%             if ll>newll
%                 newll = ll;
%                 newmix = mix;
%             end
%         end
        
        objmix{obj} = newmix;
        objll(iter,obj) = newll;
    end

%     objll(1:iter,:)
    
    disp(['iter ' num2str(iter) ': ' num2str(sum(objll(iter,:)))]);

    % need to save the GMM para to CA_chain
    CA_chain.objmix = objmix;
    objmix_prev = objmix;
    
    CA_chain.gmm_ll = sum(objll(iter,:));

    
    % estimate the posterior based on current model
    [post_pC, post_pA, post_pCX] = CA_posterior(CA_chain, 'CA_chain', ...
        train_X, train_C, train_A, MIN_POST_C);
    
    % (6.2) learn the probC & probA_C parameters
    CA_chain = learn_CA_chain_para(CA_chain, ...
        post_pC, post_pA, CA_para_epsilon);

    %%% store the learned model !!!
    CA_chain_array{iter} = CA_chain;
    
    
%     if 0 
%         % obtain all the joint data likelihood, p(X,A) p(X,C,A) p(C,A)
%         CA_chain = CA_All_LL(CA_chain, 'CA_chain', train_X, train_C, train_A);
%         %% only thing useful is the true joint p(X,A)
%         array_pxa_ll(iter) = CA_chain.pxa_ll;
%         % p(vec(A)=vec(a) | x): joint affordance
%         CA_chain.pa_x_ll = CA_cond_LL(CA_chain, 'CA_chain', ...
%             train_X, train_C, train_A, 'pA_X_group');
%         array_pa_x_ll(iter) = CA_chain.pa_x_ll;
%         
%         % p(A^k=a^k | x): individual affordance
%         [CA_chain.pa_x_ll_sep, CA_chain.err_aff_sep] = ...
%             CA_cond_LL(CA_chain, 'CA_chain', ...
%             train_X, train_C, train_A, 'pA_X_sep');
%         
%         array_pa_x_ll_sep(iter) = CA_chain.pa_x_ll_sep;
%         array_err_aff_sep(iter,:) = CA_chain.err_aff_sep;
%     end
end

% figure(1000); plot(array_pxa_ll);

