% CA-full: script to learn a CA_chain model Generatively through EM method
load train_data

CA_full_maxiter = 20;
CATENFORCE = 1;
MIN_POST_C  = 0.01;
CA_para_epsilon = 0.05;


CA_full_array = cell(1,CA_full_maxiter);


nclass = 7;
nmixture = 2;
gmm_maxiter = 10;
gmm_epsilon = 1e-6;
gmm_prior_cov = eye(dim)*1e-6; % min covariance matrix
objmix = cell(1,nclass);
for obj = 1:nclass
    ind = find(train_C == obj);
    gmmdata = train_X(ind,:);

    oldmix = gmm(dim, nmixture, 'diag');
    options = foptions; options(14) = 5; % 5 iter of k-means to init
    oldmix = gmminit(oldmix, gmmdata, options);

    [objmix{obj}, objll(obj)] = learn_category_mixture(gmmdata, [], ...
        nmixture, gmm_maxiter, gmm_epsilon, gmm_prior_cov, ...
        'diag', 'Netlab', oldmix); % cov_type, model(KPM/Netlab/Weighted)
end

displine();
disp('Initialize the CA_chain model');
CA_chain = init_CA_model('CA_chain');
CA_chain.nclass = nclass;
CA_chain.nmixture = nmixture;
CA_chain.nafford = nafford;
CA_chain.objmix = objmix;
CA_chain.ndim = dim;

disp(['Min posterior of C is : ' num2str(MIN_POST_C)]);

disp('Construct initial posterior: pC, pA');
num = size(train_X,1);
pC = zeros(num, nclass);
pA = train_A;
for i=1:num
    c = train_C(i);
    if c == -1 % if C is not observed
        %% Or even give it a smaller value, implying smaller weight %%
        pC(i,:) = (-1) * ones(1,nclass);
    else
        pC(i,c) = 1; % others are 0
    end
end

disp('Calcualte initial prob para estimation: probC, probA_C');
CA_chain = learn_CA_chain_para(CA_chain, pC, pA, CA_para_epsilon);

disp('Calculate posterior from initial model');
[post_pC, post_pA] = CA_posterior(CA_chain, 'CA_chain', ...
    train_X, train_C, train_A, MIN_POST_C);


clear weight;

% up to now it's the chain model for initialization
CA_full = CA_chain;
CA_full.type = 'CA_full';
clear CA_chain;

array_cll = zeros(1, CA_full_maxiter);
for iter = 1 : CA_full_maxiter
    % start from the most recent model: CA_full
    [fullX0, A, B, Aeq, Beq, LB, UB] = get_CA_para(CA_full, train_X);

    alldata=struct('CAmodel', CA_full, ...
        'A', train_A, 'C', train_C, 'X', train_X, ... % training data
        'LB', LB, 'UB', UB, 'fullX', fullX0, 'ind', 1:454, ... % full parameter set
        'condtype', 'pC_X',  'post_pC',post_pC);

    opt_maxiter = 5;
    options = optimset('Display','off','FunValCheck','on',...
        'TolFun',1e-6,'MaxIter',opt_maxiter,'MaxFunEvals',1e6, ...
        'LargeScale','off');

    minLLF = fun_CA_weight_cond_LL_subset(fullX0,alldata)
    
     for obj = 1:7
            % t = 28 + (obj-1)*61;
            % index = [1:6, t:t+60];
            % 2 mixutre -> tmp=61
            % 1 mixutre -> tmp=30
            tmp = nmixture*dim*2+nmixture-1;
            t = 28 + (obj-1)*tmp;
            index = [1:6, t:t+tmp-1];

        % partial vector for optimization
        alldata.ind = index;
        % the most recent fullX is in alldata.fullX
        pX0 = alldata.fullX(index);
        pA = A(index);
        pB = B;
        pLB = LB(index);
        pUB = UB(index);

        [pX1,FVAL,EXITFLAG,OUTPUT,LAMBDA,GRAD,HESSIAN] = ...
            fmincon(@(X)fun_CA_weight_cond_LL_subset(X,alldata), ...
            pX0,pA,pB,Aeq,Beq,pLB,pUB,[],options);

        % display purpose
        if obj <= nclass
            displine(); disp(['Category ' num2str(obj) '     CLL=' num2str(FVAL)]);
        end

        % put the partial optimized X into the full X
        newfullX=alldata.fullX;
        newfullX(index)=pX1;
        alldata.fullX=newfullX;
     end
     array_cll(iter) = FVAL;
     disp(['iter ' num2str(iter) ': ' num2str(array_cll(iter))]);

     % update objmix
     CA_full = put_CA_para(CA_full, alldata.fullX);

     options = foptions;
     % options(1) = 1; % display
     options(2) = 1e-2;
     options(14) = 5; % num iteration

     glm_err = zeros(nclass,nafford);
     glm_mask = zeros(nclass,nafford);

     for obj = 1:nclass
         for aff = 1:nafford
             glmdata = train_X;
             glmweight = post_pC(:,obj);

             glmtargets = train_A(:,aff);

             ind_weight = (glmweight > 0.01);
             ind_aff = (glmtargets > -1);
             ind = ind_weight & ind_aff;

             % disp([sum(ind_weight) sum(ind_aff) sum(ind) length(glmtargets)])

             glmdata = glmdata(ind,:);
             glmtargets = glmtargets(ind,:);
             glmweight = glmweight(ind,:);

             % verify if there is a need to learn the classification model
             pos = (glmtargets==1);
             neg = (glmtargets==0);
             npos = sum(pos);
             nneg = sum(neg);
             pa_c = npos / (npos+nneg);

             if (pa_c<=CA_para_epsilon) || (pa_c>=1-CA_para_epsilon)
                 % p(A|C) = 0.95 or 0.05
                 % no need to learn
                 pa_c = min(pa_c, 1-CA_para_epsilon);
                 pa_c = max(pa_c, CA_para_epsilon);

                 CA_full.affglm{obj,aff} = [];
                 CA_full.affglm{obj,aff} = pa_c;
                 % later we can test by isstruct() function

                 glm_err(obj,aff) = min(pa_c, 1-pa_c);
                 glm_mask(obj,aff) = pa_c;
             else
                 glmnet = glm(dim, 1, 'logistic');

                 % My own implementation of glmtrain_weight
                 % Note: KPM's implementation is glmtrain_weighted (paras are different)
                 % [net, options] = glmtrain_weight(net, options, x, ww, t)
                 [glmnet, opt] = glmtrain_weight(glmnet, options, ...
                     glmdata, glmweight, glmtargets);

                 y = glmfwd(glmnet, glmdata);
                 y = (y>0.5); % the positively classified ones
                 err = (y~=glmtargets);
                 glm_err(obj,aff) = sum(err.*glmweight)/sum(glmweight);

                 CA_full.affglm{obj,aff} = glmnet;
                 glm_mask(obj,aff) = -100-pa_c;
             end
         end
     end % end of GLM learning for Q3

     %%% store the learned model !!!
     CA_full_array{iter} = CA_full;
     glm_mask_array{iter}=glm_mask;
     glm_err_array{iter}=glm_err;

     if CATENFORCE == 1
         [post_pC, post_pA, post_pCX] = CA_posterior(CA_full, 'CA_full', ...
             train_X, train_C, train_A, MIN_POST_C);
     else
         empty_C = - ones(size(train_C));
         [post_pC, post_pA, post_pCX] = CA_posterior(CA_full, 'CA_full', ...
             train_X, empty_C, train_A, MIN_POST_C);
     end

    
end

total_p = 1;
full_trainErr = zeros(total_p, CA_full_maxiter, 6);
full_testErr = full_trainErr;
p = 1;

for k = 1:CA_full_maxiter
    CA_full = CA_full_array{k};
    full_trainErr(p,k,:) = get_CA_err(CA_full, train_X, train_C, train_A, 0.001);
    full_testErr(p,k,:) = get_CA_err(CA_full, test_X, test_C, test_A, 0.001);
    ll = CA_joint_LL(CA_full, 'CA_full', train_X, train_C, train_A, 'pXA');
    trainLL(p,k) = sum(ll);
    ll = CA_joint_LL(CA_full, 'CA_full', test_X, test_C, test_A, 'pXA');
    testLL(p,k) = sum(ll);
    [trainCLL(p,k), tmp] = CA_cond_LL(CA_full, 'CA_full', ...
        train_X, train_C, train_A, 'pA_X_sep');
    [testCLL(p,k), tmp] = CA_cond_LL(CA_full, 'CA_full', ...
        test_X, test_C, test_A, 'pA_X_sep');
    [trainCLL_grp(p,k), tmp] = CA_cond_LL(CA_full, 'CA_full', ...
        train_X, train_C, train_A, 'pA_X_group');
    [testCLL_grp(p,k), tmp] = CA_cond_LL(CA_full, 'CA_full', ...
        test_X, test_C, test_A, 'pA_X_group');
end

chain_trainErr = full_trainErr;
chain_testErr = full_testErr;

modeltype = 'CA_full';
batch_plot_errorbar;
