% CA-FULL: script to learn a CA_full model Generatively through EM method
clear all; clc;

% load init_mixture;
% script_learn_CA_chain_gen_EM;
% save CA_chain_gen.mat;

load CA_chain_gen.mat

MIN_POST_C  = 0.01;
CA_para_epsilon = 0.05;

disp(['Min posterior of C is : ' num2str(MIN_POST_C)]);

displine( );
rand_seed = sum(100*clock);
disp(['Seed the random number generator: ' num2str(rand_seed)]);
rand('state',rand_seed);

disp('Initialize the CA ** FULL ** model');
dispstar();
CA_full = CA_chain_array{end};
CA_full.type = 'CA_full';

CA_full.pxa_ll = [];
CA_full.pa_x_ll = [];
CA_full.pa_x_ll_sep = [];
CA_full.gmm_ll = [];

% CA_full.probC is still useful
% but probA_C is now only for approximation use

nclass = CA_full.nclass;
nafford = CA_full.nafford;
nmixture = CA_full.nmixture;
dim = CA_full.ndim;

CA_full.affglm = cell(nclass, nafford);

for obj = 1:nclass
    for aff = 1:nafford
        % glm(input_dim, output_dim, type)
        CA_full.affglm{obj,aff} = glm(dim, 1, 'logistic');
        % glm.w1 and glm.b1 are assigned randomly
    end
end

disp('Learning Genearlized Linear Models (GLM) with wIRLS');
disp(['Total number of GLM: ' num2str(nclass*nafford)]); disp(' ');
disp('1. Initialization: Use the ** CHAIN ** model to get an initial posterior q(C)'); disp(' ');
[post_pC, post_pA, post_pCX] = CA_posterior(CA_chain, 'CA_chain', ...
    train_X, train_C, train_A, MIN_POST_C);

Full_EM_maxiter = 20;
CA_full_array = cell(1,Full_EM_maxiter);
glm_mask_array = cell(1,Full_EM_maxiter);
glm_err_array = cell(1,Full_EM_maxiter);

clear weight;

objmix_prev = CA_chain.objmix;

for iter = 1:Full_EM_maxiter
    disp(['iter ' num2str(iter)]);
    % % **** Q1 = \sum_{n,c{ q_{c,n} \log p(c) ****
    % disp('Q1: Learn prior P(C) from maximizing Q1');
    assert(~any(post_pC<0));
    pC = sum(post_pC);
    pC = pC / sum(pC);
    disp('prev/new probC = '); disp([CA_full.probC; pC])
    CA_full.probC = pC;

    % % **** Q2 = \sum_{n,c} q_{c,n} \log p(x_n | c) ****
    % disp('Q2: Learn P(X|C) from maximizing Q2 by weighted GMM');
    gmm_maxiter = 10;
    gmm_epsilon = 1e-5;
    gmm_prior_cov = eye(dim)*1e-5; % min covariance matrix

    objmix = cell(1,nclass);
    objll = zeros(1,nclass);
    for obj = 1:nclass
        % get training data for each class
        gmmdata = train_X;

        % setting the weight for each data point %
        gmmweight = post_pC(:,obj);

        % filter out if weight is too small
        ind = gmmweight > 0.01;
        gmmdata = gmmdata(ind,:);
        gmmweight = gmmweight(ind,:);

        % get past mixture parameters
        oldmix =  objmix_prev{obj};

        [newmix, newll] = learn_category_mixture( ...
            gmmdata, gmmweight, ...
            nmixture, gmm_maxiter, gmm_epsilon, gmm_prior_cov, ...
            'diag', 'Weighted', ... % cov_type, model(KPM/Netlab/Weighted)
            oldmix);
        
%         for t = 1:3
%             oldmix = gmm(dim, nmixture, 'diag');
%             options = foptions;
%             options(14) = 5;	% 5 iter of k-means to init
%             oldmix = gmminit(oldmix, gmmdata, options);
%             [mix, ll] = learn_category_mixture( ...
%                 gmmdata, gmmweight, ...
%                 nmixture, gmm_maxiter, gmm_epsilon, gmm_prior_cov, ...
%                 'diag', 'Weighted', ... % cov_type, model(KPM/Netlab/Weighted)
%                 oldmix);
%             if ll>newll
%                 newll = ll;
%                 newmix = mix;
%             end
%         end
        
        objmix{obj} = newmix;
        objll(iter,obj) = newll;            
    end

    CA_full.objmix = objmix;
    prev_objmix = objmix;
    % disp(['weighted LLF: ' num2str(objll)]);
    % disp('Overwrite the GMM parameters in CA_full model: CA_full.objmix = objmix');
    % displine( );

%     CA_chain = CA_full;
%     CA_chain.type = 'CA_chain';
%     [chainLL1, chainErr1] = CA_cond_LL(CA_chain, 'CA_chain', ...
%         train_X, train_C, train_A, 'pA_X_sep')

    % % **** Q3 = \sum_{n,c} q_{n,c} \log \p(a_n|c,x_n) ****
    % disp('Q3: Learn P(A|X,C) from maximizing Q3 by weighted IRLS');
    options = foptions;
    % options(1) = 1; % display
    options(2) = 1e-2;
    options(14) = 5; % num iteration

    glm_err = zeros(nclass,nafford);
    glm_mask = zeros(nclass,nafford);

    for obj = 1:nclass
        for aff = 1:nafford
            glmdata = train_X;
            glmweight = post_pC(:,obj);

            glmtargets = train_A(:,aff);

            ind_weight = (glmweight > 0.01);
            ind_aff = (glmtargets > -1);
            % only get training data for those have higher class
            % probability and known affordance labels
            ind = ind_weight & ind_aff;

            % disp([sum(ind_weight) sum(ind_aff) sum(ind) length(glmtargets)])

            glmdata = glmdata(ind,:);
            glmtargets = glmtargets(ind,:);
            glmweight = glmweight(ind,:);

            % verify if there is a need to learn the classification model
            pos = (glmtargets==1);
            neg = (glmtargets==0);
            npos = sum(pos);
            nneg = sum(neg);
            pa_c = npos / (npos+nneg);

            if (pa_c<=CA_para_epsilon) || (pa_c>=1-CA_para_epsilon)
                % p(A|C) = 0.95 or 0.05
                % no need to learn
                pa_c = min(pa_c, 1-CA_para_epsilon);
                pa_c = max(pa_c, CA_para_epsilon);

                CA_full.affglm{obj,aff} = [];
                CA_full.affglm{obj,aff} = pa_c;
                % later we can test by isstruct() function

                glm_err(obj,aff) = min(pa_c, 1-pa_c);
                glm_mask(obj,aff) = pa_c;
            else
                % [npos nneg]
                % glmweight'

                glmnet = glm(dim, 1, 'logistic');

                % My own implementation of glmtrain_weight
                % Note: KPM's implementation is glmtrain_weighted (paras are different)
                % [net, options] = glmtrain_weight(net, options, x, ww, t)
                [glmnet, opt] = glmtrain_weight(glmnet, options, ...
                    glmdata, glmweight, glmtargets);
                % [glm, opt] = glmtrain(glm, options, ...
                %    glmdata, glmtargets);

                y = glmfwd(glmnet, glmdata);
                y = (y>0.5); % the positively classified ones
                err = (y~=glmtargets);
                glm_err(obj,aff) = sum(err.*glmweight)/sum(glmweight);

                CA_full.affglm{obj,aff} = glmnet;
                glm_mask(obj,aff) = -100;
            end
        end
    end % end of GLM learning for Q3

    CA_full_array{iter} = CA_full;
    
    glm_mask_array{iter}=glm_mask;
    glm_err_array{iter}=glm_err;
    
    % displine();
    % disp('Calculate posterior q_{n,c}');
    [post_pC, post_pA, post_pC_X] = CA_posterior(CA_full, 'CA_full', ...
        train_X, train_C, train_A, MIN_POST_C);

%     if 0
%         % for calculating the error, use a smaller min_post_c
%         % first 3 affordance, then avg afford, group afford, category
%         trainErr(iter,:) = get_CA_err(CA_full, train_X, train_C, train_A, 0.001)
%         testErr(iter,:) = get_CA_err(CA_full, test_X, test_C, test_A, 0.001)
% 
%         [trainCLL(iter), tmp] = CA_cond_LL(CA_full, 'CA_full', ...
%             train_X, train_C, train_A, 'pA_X_sep');
%         [testCLL(iter), tmp] = CA_cond_LL(CA_full, 'CA_full', ...
%             test_X, test_C, test_A, 'pA_X_sep');
% 
%         ll = CA_joint_LL(CA_full, 'CA_full', ...
%             train_X, train_C, train_A, 'pXA');
%         trainLL(iter) = sum(ll);
% 
%         ll = CA_joint_LL(CA_full, 'CA_full', ...
%             test_X, test_C, test_A, 'pXA');
%         testLL(iter) = sum(ll);
%     end
end % END of main loop

