% CA: main script file to do CA_chain experiment

clc; clear all; close all;
rand('state',sum(100*clock));

newexp = 1;
if newexp
    disp('Deleting all *.mat files, press any key to continue...');
    pause(5);
    !del *.mat
end



%%%%%%%%%% (1) Generate Data & PCA Projection %%%%%%%%%%
displine();
if isempty(ls('project.mat'))
    disp('(1.1) load all the data and put to 7 obj file (each 4000)');
    disp('loadproject calls load_define(), which has rand sampling');

    script_loadproject;

    disp('Loading files finished');
    disp('**** Saved to project.mat ****');
save project.mat
else
    disp('(1.1) loading existing project.mat');
    load project.mat
end
displine();

if isempty(ls('pca_data.mat'))
    disp('(1.2) do pca projection to a lower dimensional space, save to file');
    disp('pca_experiment() calls pca_project()');
    projdim1 = 20; % HS histo: 288 -> projdim1
    projdim2 = 10; % edge histo: 15 -> projdim2
    [featuredim, projdata1, projdata2] = pca_experiment( ...
        objdata,303,288,projdim1,projdim2);
    save pca_data.mat;
    disp('**** Save to pca_data.mat ****');
else
    disp('(1.2) loading existing pca_data.mat');
    load pca_data
end
displine();
% pca_visualize();
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





%%%%%%%%%% (2) Sample the training and testing data %%%%%%%%%%
if isempty(ls('traintestdata.mat'))
    disp('(2) Generate training/testing data from projdata1&projdata2');
    disp('Setting parameters for Gaussian Mixture Model (GMM) learning');
    disp('We use few iterations in EM since we use discriminative approach later');
    dim1 = 10;
    dim2 = 5;
    dim = dim1+dim2;
    numtrain = 50; % per object class (7 obj)
    numtest = 900; % per object class (7 obj)
    
    [train_data, test_data]= generate_traintest_data( ...
        affdata, ... % contains each individual datapoint's affordance value
        projdata1, projdata2, dim1, dim2, ...
        numtrain, numtest, ...
        0.001);  % ratio to apply to projdata2.X since they >> projdata1

    train_X = train_data.X;
    train_C = train_data.C;
    train_A = train_data.A;

    test_X = test_data.X;
    test_C = test_data.C;
    test_A = test_data.A;    
    
    save traintestdata.mat train* test* numtrain numtest
    disp('**** Save training/testing data to traintestdata.mat ****');
else
    disp('(2) loading existing traintestdata.mat');
    load traintestdata.mat
end
displine();



%%%%%%%%%% (3) Modify training data to hide unobserved variables %%%%%%%%%%
if isempty(ls('train_data.mat'))
    displine();
    disp('(3) Modify the training data to hide some variables to be unobserved');

    %%% use first 3 affordances only %%%
    nclass = 7;
    nafford = 3; % numaff is total num, nafford is number involved in experiment
    train_A = train_A(:,1:nafford);

    % store the full data
    train_A_full = train_A(:,1:nafford);
    train_C_full = train_C;
    test_A = test_A(:,1:nafford);

    subnumtrain = 50;
    nknown = 1; % number of known affordance in training data
    [subnumtrain, numtrain]
    [train_A, train_C] = modify_train_data(train_A, train_C, ...
        'fixed', subnumtrain, nknown);
    % setting category to -1 means unknown
    % setting affordance to -1 means unknown 
    disp('Sanity check of the training data (value range)');
    [unknown_C, unknown_A] = check_train_AC( ...
        train_C, train_A, train_X, nclass, nafford)
    
    disp('**** Save modified training data to train_data.mat ****');
    save train_data.mat
    displine();
else
    disp('(3) loading existing train_data.mat');
    load train_data.mat;
end
displine();
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





%%%%%%%%%% (4) Learn the initial mixture model per category %%%%%%%%%%
%%%% This implementation deals well with missing C lables %%%%
% if isempty(ls('init_mixture.mat'))
%     disp('(4) Learn a mixture model for X-C');
%     nclass = 7;
%     nmixture = 2;
%     disp([num2str(nmixture) ' mixtures for each of the ' num2str(nclass) ' classes']);
%     objmix = cell(1,nclass);
%     gmm_maxiter = 10;
%     gmm_epsilon = 1e-6;
%     gmm_prior_cov = eye(dim)*1e-6; % min covariance matrix
%     % gmm_prior_cov = eye(dim) * min(train_data.std(:)).^2;
% 
%     unknown_C = sum(train_C == -1);
%     disp(['**** ' num2str(unknown_C) ' out of ' ...
%         num2str(length(train_C)) ' sample''s C is unknown ****']);
%     
%     for obj = 1:nclass
%         ind = find(train_C == obj);
%         
%         % get training data for each class
%         gmmdata = train_X(ind,:);
%         objdata{obj} = gmmdata;
%         
%         %% setting the weight for each data point %
%         %% not needed for initial training %
%         % disp('setting the weight for each data point');
%         % weight = ones(size(gmmdata,1),1);
%         % weight = rand(size(gmmdata,1),1);
%         oldmix = gmm(dim, nmixture, 'diag');
%         options = foptions; options(14) = 5; % 5 iter of k-means to init
%         oldmix = gmminit(oldmix, gmmdata, options);
%         [objmix{obj}, objll(obj)] = learn_category_mixture(gmmdata, [], ...
%             nmixture, gmm_maxiter, gmm_epsilon, gmm_prior_cov, ...
%             'diag', 'Netlab', oldmix); % cov_type, model(KPM/Netlab/Weighted)
%     end
%     
%     disp('**** Save mixture results to init_mixture.mat ****');
%     save init_mixture.mat
% else
%     disp('(4) loading existing init_mixture.mat');
%     load init_mixture.mat
% end
% displine();
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


return;

%%%%%%%%%% (5) Learn CA_chain model with Generative Approach %%%%%%%%%%
%%%% This implementation deals well with missing C lables %%%%
!del CA_chain*.mat
if isempty(ls('CA_chain_gen.mat'))
    
    disp('(5) Learning (generative) the CA_chain model'); disp(' ');

    % learn CA_chain model using Generative method (EM)
    script_learn_CA_chain_gen_EM;


    disp('**** Save final CA_chain results to CA_chain_gen.mat ****');
    save CA_chain_gen.mat


else
    disp('(5,6) loading existing CA_chain_gen.mat');
    load CA_chain_gen.mat
end
displine();


% if isempty(ls('CA_chain_dis.mat'))
% 
%     disp('(7) Learning (discriminative) the CA_chain model'); disp(' ');
% 
%     % learn CA_chain model using Discriminative method (subset)
%     script_learn_CA_chain_dis_subset;
%     
%     disp('**** Save final CA_chain results to CA_chain_dis_subset.mat ****');
%     save CA_chain_dis_subset
% else
%     disp('(7) Loading existing CA_chain_dis_subset.mat');
%     load CA_chain_dis_subset
% end
% displine();

if isempty(ls('CA_full_gen_EM.mat'))

    disp('(8) GEN_FULL: Learning (generative) the CA_full model'); disp(' ');

    % learn CA_full model using Discriminative method (subset)
    script_learn_CA_full_gen_EM;

    disp('**** GEN_FULL: Save final CA_full results to CA_full_gen_EM.mat ****');
    save CA_full_gen_EM
else
    disp('(8) GEN_FULL: Loading existing CA_full_gen_EM.mat');
    load CA_full_gen_EM
end
displine();

titlestr={'trav','move','supp','avg','group','cat'};
figure;
for k = 1:6
    subplot(2,3,k);
    hold on;
    plot(trainErr(:,k),'g-','LineWidth',2);
    plot(testErr(:,k),'r-','LineWidth',2);
    plot(chain_trainErr(:,k),'g-');
    plot(chain_testErr(:,k),'r-');
    title(titlestr{k});
end
